/************************************************************************
	main.c

	PC Case USB LCD
    (usbGenericHidCommunication reference firmware 2_0_0_0)
    Copyright (C) 2010 Simon Inns

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

	Email: simon.inns@gmail.com

************************************************************************/

#ifndef MAIN_C
#define MAIN_C

// NOTE:  This firmware is for the PIC18F4550 and PIC18F2550 microcontrollers
// no other variation is supported (the memory map for USB is important and 
// needs to be changed for porting to other devices - it will compile, but USB
// will not work!)
//
// This firmware will only compile using the Hi-Tech C18 compiler (in pro or
// lite mode).  You cannot compile this using Microchip's C18 compiler.

// Global includes
#include <htc.h>

// Local includes
#include "usb.h"
#include "HardwareProfile.h"
#include "usb_function_hid.h"
#include "genericHID.h"
#include "debug.h"

#include "lcd.h"

// Ensure we have the correct target PIC device family
#if !defined(__18F4550) && !defined(__18F2550)
	#error "This firmware only supports either the PIC18F4550 or PIC18F2550 microcontrollers."
#endif

// PIC18F4550/PIC18F2550 fuse configuration:
// Config word 1 (Oscillator configuration)
// 20Mhz crystal input scaled to 48Mhz and configured for USB operation
__CONFIG(1, USBPLL & IESODIS & FCMDIS & HSPLL & CPUDIV1 & PLLDIV5);
// Config word 2
__CONFIG(2, VREGEN & PWRTDIS & BOREN & BORV20 & WDTDIS & WDTPS32K);
// Config word 3
__CONFIG(3, PBDIGITAL & LPT1DIS & MCLREN);
// Config word 4
__CONFIG(4, XINSTDIS & STVREN & LVPDIS & ICPORTDIS & DEBUGDIS);
// Config word 5, 6 and 7 (protection configuration)
__CONFIG(5, UNPROTECT);
__CONFIG(6, UNPROTECT);
__CONFIG(7, UNPROTECT);

// Globals

// Initialise system
static void InitialiseSystem(void)
{
	// PIC port set up --------------------------------------------------------

	// Default all pins to digital
    ADCON1 = 0x0F;

	// Configure ports as inputs (1) or outputs(0)
	TRISA = 0b00000000;
	TRISB = 0b00000000;
	TRISC = 0b00000000;
#if defined(__18F4550)
	TRISD = 0b00000000;
	TRISE = 0b00000000;
#endif

	// Clear all ports
	PORTA = 0b00000000;
	PORTB = 0b00000000;
	PORTC = 0b00000000;
#if defined(__18F4550)
	PORTD = 0b00000000;
	PORTE = 0b00000000;
#endif

	// USB set up -------------------------------------------------------------

	// USB stack and parameter initialisation
    #if defined(USE_SELF_POWER_SENSE_IO)
    tris_self_power = INPUT_PIN;
    #endif
    
    // Initialize the variable holding the handle for the last
    // transmission
    USBOutHandle = 0;
    USBInHandle = 0;
    
	// Call the stack initialisation functions
    USBDeviceInit();
}

// Process input and output
void ProcessIO(void)
{   
    // If we are not in the configured state just return
    if((USBDeviceState < CONFIGURED_STATE)||(USBSuspendControl==1))
    {
	    // We are not configured, set LED1 to off to show status
	    LED1 = OFF;
	    return;
	}
	

	// Check if data was received from the host.
    if(!HIDRxHandleBusy(USBOutHandle))
    {   
    	// Command mode    
        switch(ReceivedDataBuffer[0])
		{
			case 0x10:	// Debug information request from host
				// Copy any waiting debug text to the send data buffer
				copyDebugToSendBuffer();
			        
		        // Transmit the response to the host
                if(!HIDTxHandleBusy(USBInHandle))
				{
					USBInHandle = HIDTxPacket(HID_EP,(BYTE*)&ToSendDataBuffer[0],64);
				}
				break;
				
			// Place application specific commands here:
            	
            case 0x80:	// Clear the display
            	debugOut("Command 0x80 - Clear Display");
            	lcdClearDisplay();
            	break;
            	
            case 0x81:	// Output string
            	debugOut("Command 0x81 - Output String");
            	for (unsigned char characterNumber = 0; characterNumber < ReceivedDataBuffer[1]; characterNumber++)
            	{
	            	lcdPutch(ReceivedDataBuffer[2 + characterNumber]);
	            }	
            	break;
            	
            case 0x82: // Move cursor to x, y
            	debugOut("Command 0x82 - Move cursor");
            	lcdGoto(ReceivedDataBuffer[1], ReceivedDataBuffer[2]);
            	break;
            	
            case 0x83: // Write a raw byte to the lcd registers
            	debugOut("Command 0x83 - Write raw byte");
            	lcdWrite(ReceivedDataBuffer[1], ReceivedDataBuffer[2]);
            	break;
            	
            default:	// Unknown command received
            	debugOut("ERROR: Unknown command received from host!");
           		break;
		}
		
        // Re-arm the OUT endpoint for the next packet
        USBOutHandle = HIDRxPacket(HID_EP,(BYTE*)&ReceivedDataBuffer,64);
    }  
}   

// Main function
void main(void)
{
	// Initialise the microcontroller
    InitialiseSystem();
    
    // Initialise the debug log functions
    debugInitialise();
    
    // Initialise the LCD
    lcdInit();
    lcdGoto(0,0);
    lcdPuts("PC Case USB LCD");
    lcdGoto(0,1);
    lcdPuts("Version 1.0");
    
    // Show that we are up and running
	debugOut("PC Case USB LCD");
	debugOut("(C)2010 Simon Inns - http://www.waitingforfriday.com");
	debugOut("USB Device Initialised...");
	
    while(1)
    {
		// Check bus status and service USB interrupts.
        USBDeviceTasks();

		// Application-specific tasks.
        ProcessIO();        
    }
}

#endif
